@@ -16,19 +16,26 @@
 */
 
 #include "mergeutil.h"
+#include "logging.h"
+#include "stringutil.h"
 
 #include <KItinerary/BusTrip>
+#include <KItinerary/Event>
 #include <KItinerary/Flight>
 #include <KItinerary/JsonLdDocument>
+#include <KItinerary/RentalCar>
 #include <KItinerary/Organization>
 #include <KItinerary/Place>
 #include <KItinerary/Person>
 #include <KItinerary/Reservation>
+#include <KItinerary/Taxi>
 #include <KItinerary/TrainTrip>
 #include <KItinerary/Visit>
 
 #include <QDate>
 #include <QDebug>
+#include <QMetaObject>
+#include <QMetaProperty>
 #include <QVariant>
 
 using namespace KItinerary;
@@ -42,11 +49,15 @@ static bool equalAndPresent(const QDate &lhs, const QDate &rhs)
 {
     return lhs.isValid() && lhs == rhs;
 }
+static bool equalAndPresent(const QDateTime &lhs, const QDateTime &rhs)
+{
+    return lhs.isValid() && lhs == rhs;
+}
 
 /* Checks that @p lhs and @p rhs are not non-equal if both values are set. */
-static bool conflictIfPresent(const QString &lhs, const QString &rhs)
+static bool conflictIfPresent(const QString &lhs, const QString &rhs, Qt::CaseSensitivity caseSensitive = Qt::CaseSensitive)
 {
-    return !lhs.isEmpty() && !rhs.isEmpty() && lhs != rhs;
+    return !lhs.isEmpty() && !rhs.isEmpty() && lhs.compare(rhs, caseSensitive) != 0;
 }
 static bool conflictIfPresent(const QDateTime &lhs, const QDateTime &rhs)
 {
@@ -60,6 +71,9 @@ static bool isSameLodingBusiness(const LodgingBusiness &lhs, const LodgingBusine
 static bool isSameFoodEstablishment(const FoodEstablishment &lhs, const FoodEstablishment &rhs);
 static bool isSameTouristAttractionVisit(const TouristAttractionVisit &lhs, const TouristAttractionVisit &rhs);
 static bool isSameTouristAttraction(const TouristAttraction &lhs, const TouristAttraction &rhs);
+static bool isSameEvent(const Event &lhs, const Event &rhs);
+static bool isSameRentalCar(const RentalCar &lhs, const RentalCar &rhs);
+static bool isSameTaxiTrip(const Taxi &lhs, const Taxi &rhs);
 
 bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
 {
@@ -84,7 +98,7 @@ bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
     if (JsonLd::isA<FlightReservation>(lhs)) {
         const auto lhsRes = lhs.value<FlightReservation>();
         const auto rhsRes = rhs.value<FlightReservation>();
-        if (lhsRes.reservationNumber() != rhsRes.reservationNumber() || lhsRes.reservationNumber().isEmpty()) {
+        if (conflictIfPresent(lhsRes.reservationNumber(), rhsRes.reservationNumber()) || conflictIfPresent(lhsRes.passengerSequenceNumber(), rhsRes.passengerSequenceNumber())) {
             return false;
         }
         return isSame(lhsRes.reservationFor(), rhsRes.reservationFor());
@@ -132,7 +146,7 @@ bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
         if (lhsRes.reservationNumber() != rhsRes.reservationNumber()) {
             return false;
         }
-        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor()) && lhsRes.checkinTime().date() == rhsRes.checkinTime().date();;
+        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor()) && lhsRes.checkinTime().date() == rhsRes.checkinTime().date();
     }
     if (JsonLd::isA<LodgingBusiness>(lhs)) {
         const auto lhsHotel = lhs.value<LodgingBusiness>();
@@ -140,14 +154,49 @@ bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
         return isSameLodingBusiness(lhsHotel, rhsHotel);
     }
 
-    // restaurant reservation: sane restaurant, same booking ref, same day
+    // Rental Car
+    if (JsonLd::isA<RentalCarReservation>(lhs)) {
+        const auto lhsRes = lhs.value<RentalCarReservation>();
+        const auto rhsRes = rhs.value<RentalCarReservation>();
+        if (lhsRes.reservationNumber() != rhsRes.reservationNumber()) {
+            return false;
+        }
+        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor()) && lhsRes.pickupTime().date() == rhsRes.pickupTime().date();
+    }
+    if (JsonLd::isA<RentalCar>(lhs)) {
+        const auto lhsEv = lhs.value<RentalCar>();
+        const auto rhsEv = rhs.value<RentalCar>();
+        return isSameRentalCar(lhsEv, rhsEv);
+    }
+
+    // Taxi
+    if (JsonLd::isA<TaxiReservation>(lhs)) {
+        const auto lhsRes = lhs.value<TaxiReservation>();
+        const auto rhsRes = rhs.value<TaxiReservation>();
+        if (lhsRes.reservationNumber() != rhsRes.reservationNumber()) {
+            return false;
+        }
+        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor()) && lhsRes.pickupTime().date() == rhsRes.pickupTime().date();
+    }
+    if (JsonLd::isA<Taxi>(lhs)) {
+        const auto lhsEv = lhs.value<Taxi>();
+        const auto rhsEv = rhs.value<Taxi>();
+        return isSameTaxiTrip(lhsEv, rhsEv);
+    }
+
+    // restaurant reservation: same restaurant, same booking ref, same day
     if (JsonLd::isA<FoodEstablishmentReservation>(lhs)) {
         const auto lhsRes = lhs.value<FoodEstablishmentReservation>();
         const auto rhsRes = rhs.value<FoodEstablishmentReservation>();
         if (lhsRes.reservationNumber() != rhsRes.reservationNumber()) {
             return false;
         }
-        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor()) && lhsRes.startTime().date() == rhsRes.endTime().date();
+        auto endTime = rhsRes.endTime();
+        if (!endTime.isValid()) {
+            endTime = QDateTime(rhsRes.startTime().date(), QTime(23, 59, 59));
+        }
+
+        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor()) && lhsRes.startTime().date() == endTime.date();
     }
     if (JsonLd::isA<FoodEstablishment>(lhs)) {
         const auto lhsRestaurant = lhs.value<FoodEstablishment>();
@@ -155,6 +204,21 @@ bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
         return isSameFoodEstablishment(lhsRestaurant, rhsRestaurant);
     }
 
+    // event reservation
+    if (JsonLd::isA<EventReservation>(lhs)) {
+        const auto lhsRes = lhs.value<EventReservation>();
+        const auto rhsRes = rhs.value<EventReservation>();
+        if (lhsRes.reservationNumber() != rhsRes.reservationNumber()) {
+            return false;
+        }
+        return isSame(lhsRes.reservationFor(), rhsRes.reservationFor());
+    }
+    if (JsonLd::isA<Event>(lhs)) {
+        const auto lhsEv = lhs.value<Event>();
+        const auto rhsEv = rhs.value<Event>();
+        return isSameEvent(lhsEv, rhsEv);
+    }
+
     // tourist attraction visit
     if (JsonLd::isA<TouristAttractionVisit>(lhs)) {
         const auto l = lhs.value<TouristAttractionVisit>();
@@ -236,20 +300,129 @@ static bool isSameTouristAttraction(const TouristAttraction &lhs, const TouristA
     return lhs.name() == rhs.name();
 }
 
-bool MergeUtil::isSamePerson(const Person& lhs, const Person& rhs)
+// compute the "difference" between @p lhs and @p rhs
+static QString diffString(const QString &lhs, const QString &rhs)
 {
-    if (lhs.name().compare(rhs.name(), Qt::CaseInsensitive) == 0) {
-        return true;
+    QString diff;
+    // this is just a basic linear-time heuristic, this would need to be more something like
+    // the Levenstein Distance algorithm
+    for (int i = 0, j = 0; i < lhs.size() || j < rhs.size();) {
+        if (i < lhs.size() && j < rhs.size() && StringUtil::normalize(lhs[i]) == StringUtil::normalize(rhs[j])) {
+            ++i;
+            ++j;
+            continue;
+        }
+        if ((j < rhs.size() && (lhs.size() < rhs.size() || (lhs.size() == rhs.size() && j < i))) || i == lhs.size()) {
+            diff += rhs[j];
+            ++j;
+        } else {
+            diff += lhs[i];
+            ++i;
+        }
     }
+    return diff.trimmed();
+}
 
-    if (!equalAndPresent(lhs.familyName(), rhs.familyName(), Qt::CaseInsensitive)) {
+static bool isNameEqualish(const QString &lhs, const QString &rhs)
+{
+    if (lhs.isEmpty() || rhs.isEmpty()) {
         return false;
     }
 
-     // names from IATA BCBP can have "MS", "MR", "MRS" etc appended to the first name
-    return equalAndPresent(lhs.givenName(), rhs.givenName(), Qt::CaseInsensitive)
-        || lhs.givenName().startsWith(rhs.givenName(), Qt::CaseInsensitive)
-        || rhs.givenName().startsWith(rhs.givenName(), Qt::CaseInsensitive);
+    auto diff = diffString(lhs, rhs).toUpper();
 
-    // TODO deal with cases where on side has the name split, and the other side only has the full name
+    // remove honoric prefixes from the diff, in case the previous check didn't catch that
+    diff.remove(QLatin1String("MRS"));
+    diff.remove(QLatin1String("MR"));
+    diff.remove(QLatin1String("MS"));
+
+    // if there's letters in the diff, we assume this is different
+    for (const auto c : diff) {
+        if (c.isLetter()) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+bool MergeUtil::isSamePerson(const Person& lhs, const Person& rhs)
+{
+    return isNameEqualish(lhs.name(), rhs.name()) ||
+        (isNameEqualish(lhs.givenName(), rhs.givenName()) && isNameEqualish(lhs.familyName(), rhs.familyName()));
+}
+
+static bool isSameEvent(const Event &lhs, const Event &rhs)
+{
+    return equalAndPresent(lhs.name(), rhs.name())
+        && equalAndPresent(lhs.startDate(), rhs.startDate());
+}
+
+static bool isSameRentalCar(const RentalCar &lhs, const RentalCar &rhs)
+{
+    return lhs.name() == rhs.name();
+}
+
+static bool isSameTaxiTrip(const Taxi &lhs, const Taxi &rhs)
+{
+    //TODO verify
+    return lhs.name() == rhs.name();
+}
+
+
+static Airline mergeValue(const Airline &lhs, const Airline &rhs)
+{
+    auto a = JsonLdDocument::apply(lhs, rhs).value<Airline>();
+    // prefer the more detailed name
+    if (a.name().size() < lhs.name().size()) {
+        a.setName(lhs.name());
+    }
+    return a;
+}
+
+static QDateTime mergeValue(const QDateTime &lhs, const QDateTime &rhs)
+{
+    // prefer value with timezone
+    return lhs.isValid() && lhs.timeSpec() == Qt::TimeZone && rhs.timeSpec() != Qt::TimeZone ? lhs : rhs;
+}
+
+QVariant MergeUtil::merge(const QVariant &lhs, const QVariant &rhs)
+{
+    if (rhs.isNull()) {
+        return lhs;
+    }
+    if (lhs.isNull()) {
+        return rhs;
+    }
+    if (lhs.userType() != rhs.userType()) {
+        qCWarning(Log) << "type mismatch during merging:" << lhs << rhs;
+        return {};
+    }
+
+    auto res = lhs;
+    const auto mo = QMetaType(res.userType()).metaObject();
+    for (int i = 0; i < mo->propertyCount(); ++i) {
+        const auto prop = mo->property(i);
+        if (!prop.isStored()) {
+            continue;
+        }
+
+        auto lv = prop.readOnGadget(lhs.constData());
+        auto rv = prop.readOnGadget(rhs.constData());
+        auto mt = rv.userType();
+
+        if (mt == qMetaTypeId<Airline>()) {
+            rv = mergeValue(lv.value<Airline>(), rv.value<Airline>());
+        } else if (mt == qMetaTypeId<QDateTime>()) {
+            rv = mergeValue(lv.toDateTime(), rv.toDateTime());
+        } else if (QMetaType(mt).metaObject()) {
+            rv = merge(prop.readOnGadget(lhs.constData()), rv);
+        }
+
+        if (!rv.isNull()) {
+            prop.writeOnGadget(res.data(), rv);
+        }
+    }
+
+    return res;
 }
